package com.xypower.common;

import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Environment;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;

public class MicroPhotoContext {

    public static final String PACKAGE_NAME_MPAPP = "com.xypower.mpapp";
    public static final String PACKAGE_NAME_MPMASTER = "com.xypower.mpmaster";
    public final static String DEFAULT_MASTER_URL = "http://180.166.218.222:40101/";
    public final static String MASTER_URL_CMDID = "cmdid";
    public final static int DEFAULT_PROTOCOL = 0xFF00;

    public static class AppConfig {
        public String cmdid;
        public String server;
        public int port;
        public int protocol;
        public int networkProtocol;
        public int network; // 0: SIM1 1: SIM2 2: WIFI
        public int heartbeat;
        public int packetSize;
    }

    public static String buildAppDir(Context contxt) {

        String path = Environment.getExternalStorageDirectory().getAbsolutePath();
        if (!path.endsWith(File.separator)) {
            path += File.separator;
        }
        path += contxt.getPackageName() + File.separator;
        File pathFile = new File(path);
        if (!pathFile.exists() && !pathFile.mkdirs()) {
            return null;
        }

        return path;
    }

    public static String buildMpAppDir(Context contxt) {

        String path = Environment.getExternalStorageDirectory().getAbsolutePath();
        if (!path.endsWith(File.separator)) {
            path += File.separator;
        }

        path += PACKAGE_NAME_MPAPP + File.separator;
        File pathFile = new File(path);
        if (!pathFile.exists() && !pathFile.mkdirs()) {
            return null;
        }

        return path;
    }

    public static AppConfig getAppConfig(Context context) {

        AppConfig appConfig = new AppConfig();

        String appPath = buildAppDir(context);

        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        StringBuilder stringBuilder = null;
        try {
            File appCfgFile = new File(appPath + "data/App.json");
            if (appCfgFile.exists()) {
                inputStreamReader = new InputStreamReader(new FileInputStream(appCfgFile), "UTF-8");
                bufferedReader = new BufferedReader(inputStreamReader);
                String line;
                stringBuilder = new StringBuilder();
                while ((line = bufferedReader.readLine()) != null) {
                    stringBuilder.append(line);
                }
            }

            JSONObject jsonObject = stringBuilder == null ? new JSONObject() : new JSONObject(stringBuilder.toString());
            appConfig.cmdid = jsonObject.optString("CMDID", "");
            appConfig.server = jsonObject.optString("Server", "");
            appConfig.port = jsonObject.optInt("Port", 0);
            appConfig.protocol = jsonObject.optInt("Protocol", DEFAULT_PROTOCOL);
            appConfig.networkProtocol = jsonObject.optInt("NetworkProtocol", 0);
            appConfig.network = jsonObject.optInt("Network", 0);
            appConfig.heartbeat = jsonObject.optInt("heartbeat", 0);
            appConfig.packetSize = jsonObject.optInt("packetSize", 0);

            if (appConfig.protocol == 0) {
                appConfig.protocol = DEFAULT_PROTOCOL;
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (Exception ex) {
                }
            }
            if (inputStreamReader != null) {
                try {
                    inputStreamReader.close();
                } catch (Exception ex) {
                }
            }
        }

        return appConfig;
    }

    public static AppConfig getMpAppConfig(Context context) {

        AppConfig appConfig = new AppConfig();

        String appPath = buildMpAppDir(context);

        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        StringBuilder stringBuilder = null;
        try {
            File appCfgFile = new File(appPath + "data/App.json");
            if (appCfgFile.exists()) {
                inputStreamReader = new InputStreamReader(new FileInputStream(appCfgFile), "UTF-8");
                bufferedReader = new BufferedReader(inputStreamReader);
                String line;
                stringBuilder = new StringBuilder();
                while ((line = bufferedReader.readLine()) != null) {
                    stringBuilder.append(line);
                }
            }

            JSONObject jsonObject = stringBuilder == null ? new JSONObject() : new JSONObject(stringBuilder.toString());
            appConfig.cmdid = jsonObject.optString("CMDID", "");
            appConfig.server = jsonObject.optString("Server", "");
            appConfig.port = jsonObject.optInt("Port", 0);
            appConfig.protocol = jsonObject.optInt("Protocol", DEFAULT_PROTOCOL);
            appConfig.networkProtocol = jsonObject.optInt("NetworkProtocol", 0);
            appConfig.network = jsonObject.optInt("Network", 0);
            appConfig.heartbeat = jsonObject.optInt("heartbeat", 0);
            appConfig.packetSize = jsonObject.optInt("packetSize", 0);

            if (appConfig.protocol == 0) {
                appConfig.protocol = DEFAULT_PROTOCOL;
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (Exception ex) {
                }
            }
            if (inputStreamReader != null) {
                try {
                    inputStreamReader.close();
                } catch (Exception ex) {
                }
            }
        }

        return appConfig;
    }

    public static void saveAppConfig(Context context, AppConfig appConfig) {

        String appPath = buildAppDir(context);
        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        OutputStreamWriter outputStreamWriter = null;
        try {

            File dataPath = new File(appPath + "data/");
            if (!dataPath.exists()) {
                dataPath.mkdirs();
            }

            StringBuilder stringBuilder = new StringBuilder();
            File file = new File(dataPath, "App.json");
            if (file.exists()) {
                inputStreamReader = new InputStreamReader(new FileInputStream(file), "UTF-8");
                bufferedReader = new BufferedReader(inputStreamReader);
                String line;

                while ((line = bufferedReader.readLine()) != null) {
                    stringBuilder.append(line);
                }
            }

            JSONObject jsonObject = stringBuilder.length() > 0 ? (new JSONObject(stringBuilder.toString())) : (new JSONObject());

            jsonObject.put("CMDID", appConfig.cmdid);
            jsonObject.put("Server", appConfig.server);
            jsonObject.put("Port", appConfig.port);
            jsonObject.put("Protocol", appConfig.protocol);
            jsonObject.put("NetworkProtocol", appConfig.networkProtocol);
            jsonObject.put("Network", appConfig.network);

            if (appConfig.heartbeat > 0) {
                jsonObject.put("heartbeat", appConfig.heartbeat);
            } else {
                jsonObject.remove("heartbeat");
            }
            if (appConfig.packetSize > 0) {
                jsonObject.put("packetSize", appConfig.packetSize);
            } else {
                jsonObject.remove("packetSize");
            }

            outputStreamWriter = new OutputStreamWriter(new FileOutputStream(new File(appPath + "data/App.json")), "UTF-8");
            outputStreamWriter.write(jsonObject.toString());
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (Exception ex) {
                }
            }
            if (inputStreamReader != null) {
                try {
                    inputStreamReader.close();
                } catch (Exception ex) {
                }
            }
            if (outputStreamWriter != null) {
                try {
                    outputStreamWriter.close();
                } catch (Exception ex) {
                }

            }
        }
    }

    public static void restartMpApp(Context context) {
        /*
        Context context = MicroPhotoService.this.getApplicationContext();
        Intent intent = getPackageManager().getLaunchIntentForPackage(context.getPackageName());

        int noDelay = 1;
        intent.putExtra("noDelay", noDelay);
        PendingIntent restartIntent = PendingIntent.getActivity(context, 0, intent, 0);
        AlarmManager mgr = (AlarmManager)getSystemService(Context.ALARM_SERVICE);
        mgr.set(AlarmManager.RTC, System.currentTimeMillis() + 1000, restartIntent); // 1秒钟后重启应用
        System.exit(0);

                     */

        Intent LaunchIntent = context.getPackageManager().getLaunchIntentForPackage(PACKAGE_NAME_MPAPP);
        LaunchIntent.putExtra("noDelay", 1);
        LaunchIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        context.startActivity(LaunchIntent);
    }

    public static void saveMpAppConfig(Context context, AppConfig appConfig) {
        String appPath = buildMpAppDir(context);

        saveMpAppConfig(appPath, appConfig);
    }

    public static void saveMpAppConfig(String path, AppConfig appConfig) {

        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        OutputStreamWriter outputStreamWriter = null;
        try {

            File dataPath = new File(path + "data/");
            if (!dataPath.exists()) {
                dataPath.mkdirs();
            }

            StringBuilder stringBuilder = new StringBuilder();
            File file = new File(dataPath, "App.json");
            if (file.exists()) {
                inputStreamReader = new InputStreamReader(new FileInputStream(file), "UTF-8");
                bufferedReader = new BufferedReader(inputStreamReader);
                String line;

                while ((line = bufferedReader.readLine()) != null) {
                    stringBuilder.append(line);
                }
            }

            JSONObject jsonObject = stringBuilder.length() > 0 ? (new JSONObject(stringBuilder.toString())) : (new JSONObject());

            jsonObject.put("CMDID", appConfig.cmdid);
            jsonObject.put("Server", appConfig.server);
            jsonObject.put("Port", appConfig.port);
            jsonObject.put("Protocol", appConfig.protocol);
            jsonObject.put("NetworkProtocol", appConfig.networkProtocol);
            jsonObject.put("Network", appConfig.network);

            if (appConfig.heartbeat > 0) {
                jsonObject.put("heartbeat", appConfig.heartbeat);
            } else {
                jsonObject.remove("heartbeat");
            }
            if (appConfig.packetSize > 0) {
                jsonObject.put("packetSize", appConfig.packetSize);
            } else {
                jsonObject.remove("packetSize");
            }

            outputStreamWriter = new OutputStreamWriter(new FileOutputStream(new File(path + "data/App.json")), "UTF-8");
            outputStreamWriter.write(jsonObject.toString());
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } finally {
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (Exception ex) {
                }
            }
            if (inputStreamReader != null) {
                try {
                    inputStreamReader.close();
                } catch (Exception ex) {
                }
            }
            if (outputStreamWriter != null) {
                try {
                    outputStreamWriter.close();
                } catch (Exception ex) {
                }

            }
        }
    }

    public static String getVersionName(Context context) {
        String verName = "";
        try {
            verName = context.getPackageManager().
                    getPackageInfo(context.getPackageName(), 0).versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return verName;
    }


}
